#include "EtherCard.h"
#include "OneWire.h"
#include "DallasTemperature.h"
#include <Wire.h>
#include "Adafruit_BMP085.h"
#include <EEPROM.h>
#include "DS3231.h"

#define EEPOP            0 //23 byte

#define NumberOfDevices 2                   // Set maximum number of devices in order to dimension 

#define ONE_WIRE_BUS 7           // Data wire is plugged into digital pin 3 on the Arduino

#define REQUEST_RATE 10000 // milliseconds

DS3231 clock;
RTCDateTime dt;

// ethernet interface mac address, must be unique on the LAN
static byte mymac[] = { 0x74,0x69,0x69,0x2D,0x30,0x66 };
static byte myip[] = { 192,168,1,243 };

OneWire oneWire(ONE_WIRE_BUS);                     // Setup a oneWire instance to communicate with any OneWire devices
DallasTemperature sensors(&oneWire);               // Pass our oneWire reference to Dallas Temperature. 
byte allAddress [NumberOfDevices][8];              // Device Addresses are 8-element byte arrays.
                                                   // we need one for each of our DS18B20 sensors.

byte totalDevices;                                 // Declare variable to store number of One Wire devices
                                                   // that are actually discovered.

byte discoverOneWireDevices() {
  byte j=0;                                        // search for one wire devices and
                                                   // copy to device address arrays.
  while ((j < NumberOfDevices) && (oneWire.search(allAddress[j]))) {        
    j++;
  }
   return j                      ;                 // return total number of devices found.
}


int RefPin = A1;    // select the input pin for 
int SensorPin = A0;    // select the input pin for sensor

const int rele2Pin =  2;
const int rele3Pin =  3;
const int rele4Pin =  4;
const int rele5Pin =  5;

//const int humPin =  3;
  long t;
  word h;
  byte m;
  byte s;

static long timer=0;
Adafruit_BMP085 bmp;

byte Ethernet::buffer[700];
BufferFiller bfill;
String DSstring[NumberOfDevices];
static char xbuf[20]; //260 Для разбора строки http запроса! 350 было 300 и работало нормально
int setpoint=0;
int Pump_RATE=15000; // milliseconds


static int freeRam () {
  extern int __heap_start, *__brkval; 
  int v; 
  return (int) &v - (__brkval == 0 ? (int) &__heap_start : (int) __brkval); 
}

char * findstr(char * s, char * find, char * stop)
{
  int len,len2;
  char *x2; 

  len = strlen(find);
  x2=strstr(s,find);
  if (x2==NULL) { xbuf[0]=0x00; return(xbuf); }

  strcpy(xbuf,&x2[len]);
  len = strlen(xbuf);
  
  x2=strstr(xbuf,stop);
  len2 = strlen(x2);
  len = len-len2;

  xbuf[len] = '\0';
  s = xbuf;
  
  return s; 
}

char C2D(
char c  /**< is a character('0'-'F') to convert to HEX */
)
{
  if (c >= '0' && c <= '9')
  return c - '0';
  if (c >= 'a' && c <= 'f')
  return 10 + c -'a';
  if (c >= 'A' && c <= 'F')
  return 10 + c -'A';
  return (char)c;
}

unsigned int ATOI(char* str, int base)
{
  int num = 0;
  while (*str != 0) num = num * base + C2D(*str++);
  return num;
}




void setup () {
  Serial.begin(115200);
  delay(1000);
  Serial.println(F("WebMeteoStation server Started"));
  pinMode(rele2Pin, OUTPUT);
  pinMode(rele3Pin, OUTPUT);
  pinMode(rele4Pin, OUTPUT);
  pinMode(rele5Pin, OUTPUT);

  if (ether.begin(sizeof Ethernet::buffer, mymac, 10) == 0)
    Serial.println(F("Failed to access Ethernet controller"));
  ether.staticSetup(myip);
  sensors.begin();
  totalDevices = discoverOneWireDevices();
  for (byte i=0; i < totalDevices; i++)
  {
    DSstring[i]="";
    for(int k=7;k>=0;k--)
    {
          char msg[3];
          sprintf(msg,"%02X",allAddress[i][k]);
          DSstring[i]+=msg;
    };
   
    sensors.setResolution(allAddress[i], 12);      // and set the a to d conversion resolution of each.
  }
  bmp.begin();
  
  clock.begin();
  // Set sketch compiling time
//  clock.setDateTime(__DATE__, __TIME__);  
  // Set from UNIX timestamp
  // clock.setDateTime(1397408400);
  // Manual (YYYY, MM, DD, HH, II, SS
  // clock.setDateTime(2014, 4, 13, 19, 21, 00);
  Serial.println("setup");

}

static word homePage() {
  float tempC = 0;
  float bmpT,bmpP;
  int t1,t2,bmpT1,bmpT2,bmpP1,bmpP2;
  bmpT = bmp.readTemperature();
  bmpT1 = (int)bmpT;
  bmpT2 = (bmpT-bmpT1)*100;
  bmpP = bmp.readPressure()/133.3;
  bmpP1 = (int)bmpP;
  bmpP2 = (bmpP-bmpP1)*100;
  sensors.requestTemperatures();                // Initiate  temperature request to all devices
  String temp;
  if (totalDevices>0){
    for (byte i=0; i < totalDevices; i++) {
      tempC = sensors.getTempC(allAddress[i]);           // read the device at addr.
      t1 = (int)tempC;
      t2 = (tempC-t1)*100;
    }
  }
  else {tempC = 0; t1 = 0; t2 = 0;}
  dt = clock.getDateTime();
  bfill = ether.tcpOffset();
  bfill.emit_p(PSTR(
    "HTTP/1.0 200 OK\r\n"
    "Content-Type: text/html\r\n"
//    "Pragma: no-cache\r\n"
    "\r\n"
//    "<meta http-equiv='refresh' content='10'/>"
    "<title>WSM server</title><body align=center>"
    "<h1>BMP180 T=$D.$D P=$D.$D<br>DS18B20 $D.$D<br>$D$D:$D$D:$D$D"),
    bmpT1, bmpT2, bmpP1, bmpP2, 
    t1, t2, 
    h/10, h%10, m/10, m%10, s/10, s%10);
    
  bfill.emit_p(PSTR("<p>$S"), clock.dateFormat("d-m-Y H:i:s", dt));
 
  bfill.emit_p(PSTR("</h1><br><a href=\"/d"
     "\" >Relay Control</a> <br><br>($D bytes free)</body></html>"),freeRam());
  return bfill.position();
}
static word http404(void)
{
  bfill = ether.tcpOffset();
  bfill.emit_p(PSTR(
    "HTTP/1.0 404 Not Found\r\n"
    "Content-Type: text/html\r\n\r\nErr"));
  return bfill.position();
}
static word httpOK(int vall)
{
   bfill = ether.tcpOffset();
   bfill.emit_p(PSTR(
    "HTTP/1.0 200 OK\r\n"
    "Content-Type: text/html\r\n"
    "Pragma: no-cache\r\n"
    "\r\n"
    "<title>Plant server</title>" 
    "<h1>New value is $D</h1>"), vall);
   return bfill.position();
}


static word printDigital()
{
  char D[6];
  D[2] = digitalRead(rele2Pin);  
  D[3] = digitalRead(rele3Pin);  
  D[4] = digitalRead(rele4Pin);  
  D[5] = digitalRead(rele5Pin);  
  bfill = ether.tcpOffset();
  bfill.emit_p(PSTR( 
    "HTTP/1.1 200 OK\r\n"
    "Content-type:text/html\r\n\r\n"
    "<html><body align=center>"
//   <head><title>WSM server</title></head><body align=center>"
    "<h1>Digital output</h1>\r\n"));
    for (char i=2; i<6; i++){
      bfill.emit_p(PSTR( 
      "D$D=$D <button onclick=\"location.href='/r$D"
      "=1'\">ON</button> <button onclick=\"location.href='/r$D"
      "=0'\">OFF</button><br>" 
      ),i,D[i],i,i);
    }
  bfill.emit_p(PSTR( 
    "<p><p><a href=\"/d"
    "\">refresh</a><br><a href=\"/"
    "\">home</a>"));     
  return bfill.position();
}

void loop () {
  word len = ether.packetReceive();
  word pos = ether.packetLoop(len);
  char tmpstr[10];
  char tmpbuf[15];
  
  t = millis() / 1000;
  h = t / 3600;
  m = (t / 60) % 60;
  s = t % 60;
 
//  if (pos)  ether.httpServerReply(homePage()); // send web page data
   if (pos){  // check if valid tcp data is received
//    bfill = ether.tcpOffset();
    char* data = (char *) Ethernet::buffer + pos;
//    Serial.println(data);

    if (strncmp("GET / ", data, 6) == 0){
      ether.httpServerReply(homePage()); // send web page data
    }
    else if (strncmp("GET /f", data, 6) == 0)
      ether.httpServerReply(http404()); // send web page data
    else if (strncmp("GET /d", data, 6) == 0){
      ether.httpServerReply(printDigital()); // send web page data   
    }   
    else if (strncmp("GET /h", data, 6) == 0){
      memcpy(tmpbuf,data,12);
      strcpy((char *)tmpstr,findstr((char *)tmpbuf,"GET /h="," "));
      setpoint=ATOI(tmpstr,10);
      EEPROM.write(EEPOP, setpoint);
      EEPROM.write(EEPOP+1, setpoint>>8);
      ether.httpServerReply(httpOK(setpoint)); // send web page data
    } 
    else if (strncmp("GET /p", data, 6) == 0){
      memcpy(tmpbuf,data,12);
      strcpy((char *)tmpstr,findstr((char *)tmpbuf,"GET /p="," "));
      Pump_RATE=ATOI(tmpstr,10);
      EEPROM.write(EEPOP+2, Pump_RATE);
      EEPROM.write(EEPOP+3, Pump_RATE>>8);
      ether.httpServerReply(httpOK(Pump_RATE)); // send web page data
    }
    else if (strncmp("GET /r2", data, 7) == 0){
      memcpy(tmpbuf,data,12);
      strcpy((char *)tmpstr,findstr((char *)tmpbuf,"GET /r2="," "));
      Pump_RATE=ATOI(tmpstr,10);
      if (Pump_RATE!=0) digitalWrite(rele2Pin, true); else digitalWrite(rele2Pin, false);
      ether.httpServerReply(printDigital()); // send web page data
    }      
    else if (strncmp("GET /r3", data, 7) == 0){
      memcpy(tmpbuf,data,12);
      strcpy((char *)tmpstr,findstr((char *)tmpbuf,"GET /r3="," "));
      Pump_RATE=ATOI(tmpstr,10);
      if (Pump_RATE!=0) digitalWrite(rele3Pin, true); else digitalWrite(rele3Pin, false);
      ether.httpServerReply(printDigital()); // send web page data
    }      
    else if (strncmp("GET /r4", data, 7) == 0){
      memcpy(tmpbuf,data,12);
      strcpy((char *)tmpstr,findstr((char *)tmpbuf,"GET /r4="," "));
      Pump_RATE=ATOI(tmpstr,10);
      if (Pump_RATE!=0) digitalWrite(rele4Pin, true); else digitalWrite(rele4Pin, false);
      ether.httpServerReply(printDigital()); // send web page data
    }      
    else if (strncmp("GET /r5", data, 7) == 0){
      memcpy(tmpbuf,data,12);
      strcpy((char *)tmpstr,findstr((char *)tmpbuf,"GET /r5="," "));
      Pump_RATE=ATOI(tmpstr,10);
      if (Pump_RATE!=0) digitalWrite(rele5Pin, true); else digitalWrite(rele5Pin, false);
      ether.httpServerReply(printDigital()); // send web page data
    }            
  
  }
}
